import numpy as np
import cv2


def run_episode(env, policy, with_render=False):
    env.reset()

    done = False
    reward = 0
    timestep = 1
    while not done:
        _, r, done, _ = env.step(policy(timestep))
        if with_render:
            img = env.render()
            cv2.imshow('frame', img)
            cv2.waitKey()
        reward += r
        timestep += 1

    return reward


def single_freq_policy(freq):
    return lambda t: t%freq==0


class two_freq_policy(object):

    def __init__(self, f1, f2):
        self.f1 = f1
        self.f2 = f2
        self.last_f = 0
        self.last_t = 0

    def __call__(self, t):
        a = (t-self.last_t)%self.f1==0 if not self.last_f else (t-self.last_t)%self.f2==0
        if a:
            self.last_t = t
            self.last_f = not self.last_f
        return int(a)


def get_non_dominated(solutions):
    # compute pareto front
    is_efficient = np.ones(solutions.shape[0], dtype=bool)
    for i, c in enumerate(solutions):
        if is_efficient[i]:
            # Remove dominated points, will also remove itself
            is_efficient[is_efficient] = np.any(solutions[is_efficient] > c, axis=1)
            # keep this solution as non-dominated
            is_efficient[i] = 1

    return solutions[is_efficient]


if __name__ == '__main__':
    import gym
    import gym_sumo
    from gym.wrappers.time_limit import TimeLimit

    env = gym.make('CrossroadSumo-v0')
    env = TimeLimit(env, max_episode_steps=100)

    # single_freq = []
    # for f in range(1, 105, 5):
    #     r = run_episode(env, single_freq_policy(f))
    #     single_freq.append((f, r))

    two_freq = []
    for i in range(1, 105, 10):
        for j in range(1, 105, 10):
            r = run_episode(env, two_freq_policy(i, j))
            two_freq.append((i, j, r))
    
    nd = get_non_dominated(np.array([r[2] for r in two_freq]))
    print(nd)

    import matplotlib.pyplot as plt
    plt.figure()
    plt.plot(nd[:,1], nd[:,0], 'o')
    plt.show()